#!/usr/bin/env python
import os, sys
import time, subprocess

# Set solver paths
# Comment it if you do not run it.
solvers = {}
solvers['CDN-NS'] = './liblinear/train-ns'
solvers['CDN'] = './liblinear/train'
solvers['TRON'] = './tron/tron-l2svm/train'
#solvers['BMRM'] = './bmrm-2.2/linear-bmrm/linear-bmrm-train'

# The best parameter C is selected by 5-fold cross-validation.
# Comment it if you do not run it.
bestC = {}
#bestC[('leu.scale','svm')] = 16.0
#bestC[('duke.scale','svm')] = 16.0
bestC[('a9a','svm')] = 0.5
bestC[('real-sim','svm')] = 1.0
bestC[('news20.binary','svm')] = 64.0
#bestC[('rcv1_test.binary','svm')] = 1.0
#bestC[('train308.scale','svm')] = 1.0
#bestC[('leisure.scale','svm')] = 1.0

# Set runtime limits (in seconds)
runtime = {}
runtime['leu.scale'] = 50
runtime['duke.scale'] = 50
runtime['a9a'] = 100
runtime['real-sim'] = 100
runtime['news20.binary'] = 10000
runtime['rcv1_test.binary'] = 1000
runtime['train308.scale'] = 5000
runtime['leisure.scale'] = 50000

# Keep the order of all data sets
all_data = ['leu.scale','duke.scale','a9a','real-sim','news20.binary',\
			'rcv1_test.binary','train308.scale','leisure.scale']

def run(subp, rtime):
	start = time.time()
	stop = start + rtime
	while True:
		out = subp.poll()
		if out is not None:
			return out
		if time.time() >= stop:
			print 'Already reach time limit (%s)!!' %rtime
			print 'Ready to KILL the job (%s)!!' %subp.pid
			os.kill(subp.pid, 9)
		time.sleep(3)

def compare(data):
	# Check to run or not
	if not bestC.has_key((data,'svm')):
		print '---------------------------------------'
		print data, 'Skpped!!'
		print
		return
	# Check if data exist
	train_data = 'data/%s.train' %data
	test_data = 'data/%s.test' %data
	if not os.path.exists(train_data) or\
	   not os.path.exists(test_data):
		print '---------------------------------------'
		print train_data, 'or', test_data, 'do not exist!!'
		print
		return
	if not runtime.has_key(data):
		runtime[data] = 1000

	cmd = 'wc -l %s' %train_data
	fp = os.popen(cmd)
	no_inst = int(fp.readline().strip().split()[0])
	fp.close()

	C = bestC[(data,'svm')]
	lambda1 = 1.0/C
	lambda2 = 1.0/C/no_inst
	print '---------------------------------------'
	print train_data, no_inst
	print C, lambda1, lambda2
	print '---------------------------------------'

	#CDN-NS
	if solvers.has_key('CDN-NS'):
		solver = solvers['CDN-NS']
		eps = 1e-6
		log = './logSVM/CDN-NS_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s -s 5 -c %s -e %s %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data])
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'CDN-NS Skipped!!'
	else:
		print 'CDN-NS Skipped!!'

	#CDN
	if solvers.has_key('CDN'):
		solver = solvers['CDN']
		eps = 16.0
		log = './logSVM/CDN_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s -s 5 -c %s -e %s -n 40 -g 1.5 %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data]*5)
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'CDN Skipped!!'
	else:
		print 'CDN Skipped!!'

	#TRON
	if solvers.has_key('TRON'):
		solver = solvers['TRON']
		eps = 1e-8
		log = './logSVM/TRON_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s -s 0 -c %s -e %s %s %s'\
			%(solver,C,eps,train_data,test_data)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			os.system('%s > %s' %(cmd,tmplog))
#			fobj = open(tmplog, 'w')
#			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
#			run(subp, runtime[data])
#			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'TRON Skipped!!'
	else:
		print 'TRON Skipped!!'

	#BMRM
	if solvers.has_key('BMRM'):
		solver = solvers['BMRM']
		eps = 1e-3
		max_iter = 2000
		config = './tmp/BMRM_%s_%s.config' %(data,C)
		model = './tmp/BMRM_%s_%s.model' %(data,C)
		content = \
"""
string Solver.type BMRM
int BMRM.verbosity 2
int BMRM.maxNumOfIter %s
double BMRM.lambda %s
double BMRM.relEpsilonTol %s
//
int InnerSolver.verbosity 0
string BMRM.innerSolverType L1N1_GLPK
string L1N1_GLPK.LPSolverType SIMPLEX
int L1N1_GLPK.gradIdleAge 5
bool L1N1_GLPK.removeAllIdleGradients true
//
int Loss.verbosity 3
string Loss.lossFunctionType SQUARED_HINGE
bool Loss.nonNegative true
//
int Data.verbosity 1
bool Data.bias false
string Data.format VECTOR_LABEL_VECTOR_FEATURE
string Data.featureTrainFile %s.bmrm.x
string Data.labelTrainFile %s.bmrm.y
string Data.featureTestFile %s.bmrm.x
string Data.labelTestFile %s.bmrm.y
//
string Model.modelFile %s
""" %(max_iter,lambda2,eps,train_data,train_data,test_data,test_data,model)

		fp = open(config, 'w')
		fp.write(content)
		fp.close()

		log = './logSVM/BMRM_%s_%s.log' %(data,C)
		tmplog = '%s.tmp' %log
		cmd = '%s %s' %(solver,config)
		if os.path.exists(solver) and not os.path.exists(log):
			print cmd
			fobj = open(tmplog, 'w')
			subp = subprocess.Popen(cmd.split(), shell=False, stdout=fobj)
			run(subp, runtime[data])
			fobj.close()
			os.system('mv %s %s' %(tmplog,log))
		else:
			print 'BMRM Skipped!!'
	else:
		print 'BMRM Skipped!!'


if __name__ == '__main__':
	data_list = []
	for data in all_data:
		if (data,'svm') in bestC.keys():
			data_list.append(data)
	for data in data_list:
		compare(data)

