#!/usr/bin/env Rscript
.libPaths(c("./lib",.libPaths()))
library("SparseM")
library("Matrix")
library("e1071")
library("glmnet")

C = 1.0
eps = 1.0e-4
iter = 100
data = "./data/a9a"
traindata = "./data/a9a.train"
testdata = "./data/a9a.test"
################
eps_loop = FALSE
eps_g = 2.0
eps_n = 1
################

args = commandArgs(TRUE)
if(length(args) != 0) {
	for(i in 1:length(args)) {
		eval(parse(text=args[[i]]))
	}
	traindata = paste(data,"train",sep=".")
	testdata = paste(data,"test",sep=".")
} else {
	print("R --vanilla --slave --args C=1.0 eps=1.0e-4 eps_loop=FALSE eps_g=2.0 eps_n=1 iter=100 data='./data/a9a' < runGLMNET.R > runGLMNET.Rout")
	quit()
}
print("Arguments:")
cat("C",C,"\n")
cat("eps",eps,"\n")
cat("iter",iter,"\n")
cat("data",data,"\n")
cat("eps_loop",eps_loop,"\n")
cat("eps_g",eps_g,"\n")
cat("eps_n",eps_n,"\n")

#load training data
svm = read.matrix.csr(traindata)
X = svm[["x"]]
X = as(X,"Matrix")
dimX = dim(X)
Y = svm[["y"]]
Y = c(Y)
lenY = length(Y)
print("Training Data:")
cat("traindata",traindata,"\n")
cat("dimX",dimX,"\n")
cat("lenY",lenY,"\n")
rm(svm)

#load testing data
svm = read.matrix.csr(testdata,fac=TRUE,ncol=dimX[2])
XX = svm[["x"]]
XX = as(XX,"Matrix")
dimXX = dim(XX)
YY = svm[["y"]]
YY = c(YY)
lenYY = length(YY)
print("Testing Data:")
cat("testdata",testdata,"\n")
cat("dimXX",dimXX,"\n")
cat("lenYY",lenYY,"\n")
rm(svm)

#set parameters
lambda = 1.0/C/lenY
print("Parameter:")
cat("C",C,"#",lenY,"\n")
cat("lambda",lambda,"\n")

#train
print("Training ...")
if(!eps_loop) {
	print("eps_loop=FALSE!!")
	lrmodel = glmnet(X,Y,XX,YY,family="binomial",alpha=1.0,lambda=c(lambda),standardize=FALSE,thresh=eps,maxit=iter,HessianExact=TRUE)
} else {
	print("eps_loop=TRUE!!")
	cat("eps_g",eps_g, "eps_n",eps_n,"\n")
	for(i in 1:eps_n) {
		cat("i",i, "eps",eps,"\n")
		lrmodel = glmnet(X,Y,XX,YY,family="binomial",alpha=1.0,lambda=c(lambda),standardize=FALSE,thresh=eps,maxit=iter,HessianExact=TRUE)
		eps = eps / eps_g
	}
}
print(lrmodel)


