function draw(outfile, infiles, linestyles, legends, ylabelcode, axistype, xmin, xmax, ymin, ymax, ylevel)
%
% [ylabelcode] = 0 / 1 / 2 / 3 / 4
%			0 for Relative Function Value Difference
%			1 for Testing Accuracy
%			2 for Number of Nonzeros
%			3 for Function Value
%			4 for Norm of Gradient
% [axistype] = 0 / 1 / 2 / 3
%			0 for plot
%			1 for semilogx
%			2 for semilogy
%			3 for loglog
% [xmin,xmax]: range of x-axis
% [ymin,ymax]: range of y-axis
%

Xs = {};
Ys = {};
xlabelname = '';
ylabelname = '';
legendlocation = '';

%check input arguments
if (nargin ~= 11)
	error('draw2(outfile, infiles, linestyles, legends, ylabelcode, axistype, xmin, xmax, ymin, ymax, ylevel)');
end

if (ylabelcode == 0)
	ylabelname = 'Relative Function Value Difference';
	%legendlocation = 'NorthEast';
	legendlocation = 'SouthWest';
elseif (ylabelcode == 1)
	ylabelname = 'Testing Accuracy';
	legendlocation = 'SouthEast';
elseif (ylabelcode == 2)
	ylabelname = 'Number of Nonzeros';
	legendlocation = 'NorthEast';
elseif (ylabelcode == 3)
	ylabelname = 'Function Value';
	legendlocation = 'NorthEast';
elseif (ylabelcode == 4)
	ylabelname = 'Norm of Gradient';
	legendlocation = 'SouthWest';
else
	error('Unknown [ylabelcode]');
end

if (axistype>=0 || axistype<=3)
	if (axistype==0 || axistype==2)
		xlabelname = 'Training Time (sec)';
	else
		xlabelname = 'Log-scaled Training Time (sec)';
	end
else
	error('Unknown [axistype]');
end

%read data from [infiles]
for i = 1:length(infiles)
	fprintf('%s\n',infiles{i});
	[Xs{i}, Ys{i}] = textread(infiles{i}, '%f %f');
end

%draw plots
figure;
for i = 1:length(infiles)
	set(gca, 'fontsize', 16);
	Xi = Xs{i};
	Yi = Ys{i};
	if (axistype == 0)
		plot(Xi,Yi,linestyles{i},'LineWidth',3,'MarkerSize',6);
	elseif (axistype == 1)
		semilogx(Xi,Yi,linestyles{i},'LineWidth',3,'MarkerSize',6);
	elseif (axistype == 2)
		semilogy(Xi,Yi,linestyles{i},'LineWidth',3,'MarkerSize',6);
	elseif (axistype == 3)
		loglog(Xi,Yi,linestyles{i},'LineWidth',3,'MarkerSize',6);
	end
	hold on;
end

if (ylevel ~= 0)
	hline = refline(0,ylevel);
	set(hline,'Color','k','LineStyle',':','LineWidth',2);
end
hold off;

if (xmin ~= xmax)
	xlim([xmin xmax]);
end
if (ymin ~= ymax)
	ylim([ymin ymax]);
end

legend(legends, 'Location', legendlocation);
xlabel(xlabelname, 'FontSize', 18);
ylabel(ylabelname, 'FontSize', 18);

%print('-dpng', '-r200', '-loose', outfile);
print('-depsc', '-r200', '-loose', outfile);


